# coding: latin-1
# Copyright (c) 2014 Dirk Baechle.
# www: https://bitbucket.org/dirkbaechle/cmake2scons
# mail: dl9obn AT darc.de
#
# Permission is hereby granted, free of charge, to any person obtaining
# a copy of this software and associated documentation files (the
# "Software"), to deal in the Software without restriction, including
# without limitation the rights to use, copy, modify, merge, publish,
# distribute, sublicense, and/or sell copies of the Software, and to
# permit persons to whom the Software is furnished to do so, subject to
# the following conditions:
#
# The above copyright notice and this permission notice shall be included
# in all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY
# KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
# WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
# NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
# LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
# OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
# WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
#

# Handlers for the different CMake commands


def noArgs(handler, opts):
    msg = "Warning: %s called without any arguments!" % handler
    msg += " (%s)" % opts.get('file', 'unknown')
    print msg


def ifReplace(x):
    if x == "AND":
        return "and"
    elif x == "OR":
        return "or"
    elif x == "STREQUAL":
        return "=="
    elif x == "NOT":
        return "not"
    return x


def handleIf(args, fout, env, opts):
    if fout:
        if args:
            # filter arguments
            fargs = [ifReplace(x) for x in args]
            fout.write(" "*(opts['indent'])+"if (%s):\n" % ' '.join(fargs))
        else:
            noArgs("handleIf")
            fout.write(" "*(opts['indent'])+"if 0:\n")

    opts['indent'] = opts['indent'] + 4
    return env, opts


def handleElse(args, fout, env, opts):
    if fout:
        fout.write(" "*(opts['indent']-4)+"else:\n")

    return env, opts


def handleEndif(args, fout, env, opts):
    if fout:
        fout.write("\n")

    opts['indent'] = opts['indent'] - 4
    return env, opts


def handleAddSubdirectory(args, fout, env, opts):
    if fout:
        if args:
            fout.write(" "*(opts['indent']) +
                       "envLocal.SConscript('%s/SConscript')\n" % args[0])
        else:
            noArgs("handleAddSubdirectory", opts)

    return env, opts


def handleAddToolSubdirectory(args, fout, env, opts):
    return handleAddSubdirectory(args, fout, env, opts)


def handleAddExecutable(args, fout, env, opts):
    if fout:
        if len(args) > 1:
            if len(args) > 2:
                fout.write(" "*(opts['indent']) +
                           'envLocal.Program("%s", Split("""%s"""))\n' %
                           (args[0], ' '.join(args[1:])))
            else:
                fout.write(" "*(opts['indent']) +
                           'envLocal.Program("%s", "%s")\n' %
                           (args[0], args[1]))
        else:
            noArgs("handleAddExecutable", opts)

    return env, opts


def handleAddTool(args, fout, env, opts):
    return handleAddExecutable(args, fout, env, opts)


def handleIncludeDirectories(args, fout, env, opts):
    if fout:
        if args:
            fout.write(" "*(opts['indent']) +
                       "envLocal.Append(CPPPATH=['%s'])\n" % "', '".join(args))
        else:
            noArgs("handleIncludeDirectories", opts)

    return env, opts


def handleLinkDirectories(args, fout, env, opts):
    if fout:
        if args:
            fout.write(" "*(opts['indent']) +
                       "envLocal.Append(LIBPATH=['%s'])\n" % "', '".join(args))
        else:
            noArgs("handleLinkDirectories", opts)

    return env, opts


def handleSet(args, fout, env, opts):
    if fout:
        if args:
            if len(args) > 1:
                if args[0].endswith("_LINK_COMPONENTS"):
                    fout.write(" "*(opts['indent']) +
                               "envLocal.Append(LIBS=['%s'])\n" %
                               "', '".join(args[1:]))
                else:
                    # Set a global option
                    fout.write(" "*(opts['indent']) +
                               "env['%s'] = '%s'\n" %
                               (args[0], args[1].strip('"')))
            else:
                # Set a global option empty
                fout.write(" "*(opts['indent'])+"env['%s'] = ''\n" % args[0])
        else:
            noArgs("handleSet", opts)

    return env, opts


def handleTargetLinkLibraries(args, fout, env, opts):
    if fout:
        if args:
            fout.write(" "*(opts['indent']) +
                       "envLocal.Append(LIBS=['%s'])\n" % "', '".join(args))
        else:
            noArgs("handleTargetLinkLibraries", opts)

    return env, opts


def handleAddLibrary(args, fout, env, opts):
    if fout:
        if len(args) > 1:
            if len(args) > 2:
                fout.write(" "*(opts['indent']) +
                           'envLocal.Library("%s", Split("""%s"""))\n' %
                           (args[0], ' '.join(args[1:])))
            else:
                fout.write(" "*(opts['indent']) +
                           'envLocal.Library("%s", "%s")\n' %
                           (args[0], args[1]))
        else:
            noArgs("handleAddLibrary", opts)

    return env, opts


def handleAddLoadableModule(args, fout, env, opts):
    if fout:
        if len(args) > 1:
            if len(args) > 2:
                fout.write(" "*(opts['indent']) +
                           'envLocal.SharedLibrary("%s", Split("""%s"""))\n' %
                           (args[0], ' '.join(args[1:])))
            else:
                fout.write(" "*(opts['indent']) +
                           'envLocal.SharedLibrary("%s", "%s")\n' %
                           (args[0], args[1]))
        else:
            noArgs("handleAddLoadableModule", opts)

    return env, opts


def handleAppendTargetCFlags(args, fout, env, opts):
    if fout:
        if args:
            fout.write(" "*(opts['indent']) +
                       "envLocal.Append(CXXFLAGS=['%s'])\n" %
                       "', '".join(args))
        else:
            noArgs("handleAppendTargetCFlags", opts)

    return env, opts


def handleSetTargetLinkFlags(args, fout, env, opts):
    if fout:
        if args:
            fout.write(" "*(opts['indent']) +
                       "envLocal.Replace(LDFLAGS=['%s'])\n" %
                       "', '".join(args))
        else:
            noArgs("handleSetTargetLinkFlags", opts)

    return env, opts


def handleSetTargetCompileFlags(args, fout, env, opts):
    if fout:
        if args:
            fout.write(" "*(opts['indent']) +
                       "envLocal.Replace(CXXFLAGS=['%s'])\n" %
                       "', '".join(args))
        else:
            noArgs("handleSetTargetCompileFlags", opts)

    return env, opts
